Index: base/config.c
===================================================================
RCS file: /cvsroot/nagios/nagios/base/config.c,v
retrieving revision 1.115
diff -U3 -r1.115 config.c
--- base/config.c	13 Apr 2008 18:34:27 -0000	1.115
+++ base/config.c	10 Aug 2008 17:22:49 -0000
@@ -210,6 +210,10 @@
 extern int              debug_verbosity;
 extern unsigned long    max_debug_file_size;
 
+extern int		create_orphan_hostgroup;
+extern int		create_orphan_servicegroup;
+
+
 
 
 /******************************************************************/
@@ -1282,6 +1286,12 @@
 		else if(!strcmp(variable,"external_command_buffer_slots"))
 			external_command_buffer_slots=atoi(value);
 
+		else if(!strcmp(variable,"create_orphan_hostgroup"))
+			create_orphan_hostgroup=(atoi(value)>0)?TRUE:FALSE;
+
+		else if(!strcmp(variable,"create_orphan_servicegroup"))
+			create_orphan_servicegroup=(atoi(value)>0)?TRUE:FALSE;
+
 		/*** AUTH_FILE VARIABLE USED BY EMBEDDED PERL INTERPRETER ***/
 		else if(!strcmp(variable,"auth_file")){
 
Index: base/nagios.c
===================================================================
RCS file: /cvsroot/nagios/nagios/base/nagios.c,v
retrieving revision 1.156
diff -U3 -r1.156 nagios.c
--- base/nagios.c	25 Jun 2008 18:16:30 -0000	1.156
+++ base/nagios.c	10 Aug 2008 17:22:50 -0000
@@ -234,6 +234,9 @@
 FILE            *command_file_fp;
 int             command_file_created=FALSE;
 
+int		create_orphan_hostgroup=FALSE;
+int		create_orphan_servicegroup=FALSE;
+
 
 extern contact	       *contact_list;
 extern contactgroup    *contactgroup_list;
Index: cgi/cgiutils.c
===================================================================
RCS file: /cvsroot/nagios/nagios/cgi/cgiutils.c,v
retrieving revision 1.81
diff -U3 -r1.81 cgiutils.c
--- cgi/cgiutils.c	23 Jun 2008 20:47:44 -0000	1.81
+++ cgi/cgiutils.c	10 Aug 2008 17:22:57 -0000
@@ -69,6 +69,9 @@
 char            *splunk_url=NULL;
 int             lock_author_names=TRUE;
 
+int             show_orphan_hosts_hostgroup=FALSE;
+int             show_orphan_services_servicegroup=FALSE;
+
 extern time_t   program_start;
 extern int      nagios_pid;
 extern int      daemon_mode;
@@ -405,6 +408,12 @@
 
 		else if(!strcmp(var,"lock_author_names"))
 			lock_author_names=(atoi(val)>0)?TRUE:FALSE;
+
+		else if(!strcmp(var,"show_orphan_hosts_hostgroup"))
+			show_orphan_hosts_hostgroup=(atoi(val)>0)?TRUE:FALSE;
+
+		else if(!strcmp(var,"show_orphan_services_servicegroup"))
+			show_orphan_services_servicegroup=(atoi(val)>0)?TRUE:FALSE;
  	        }
 
 	/* free memory and close the file */
Index: common/objects.c
===================================================================
RCS file: /cvsroot/nagios/nagios/common/objects.c,v
retrieving revision 1.117
diff -U3 -r1.117 objects.c
--- common/objects.c	13 Apr 2008 18:34:27 -0000	1.117
+++ common/objects.c	10 Aug 2008 17:23:21 -0000
@@ -57,13 +57,25 @@
 
 skiplist *object_skiplists[NUM_OBJECT_SKIPLISTS];
 
+void create_orphan_hosts_hostgroup();
+void create_orphan_services_servicegroup();
+
 
 #ifdef NSCORE
 int __nagios_object_structure_version=CURRENT_OBJECT_STRUCTURE_VERSION;
 extern int use_precached_objects;
+extern int create_orphan_hostgroup;
+extern int create_orphan_servicegroup;
 #endif
 
 
+#ifdef NSCGI
+extern int show_orphan_hosts_hostgroup;
+extern int show_orphan_services_servicegroup;
+#endif
+
+
+
 
 /******************************************************************/
 /******* TOP-LEVEL HOST CONFIGURATION DATA INPUT FUNCTION *********/
@@ -85,6 +97,22 @@
 		return ERROR;
 #endif
 
+#ifdef NSCORE
+        /* create a new hostgroup with orphaned hosts and a servicegroup with orphaned services */
+	if (create_orphan_hostgroup==TRUE)
+		create_orphan_hosts_hostgroup();
+	if (create_orphan_servicegroup==TRUE)
+		create_orphan_services_servicegroup();
+#endif
+
+#ifdef NSCGI
+        /* create a new hostgroup with orphaned hosts and a servicegroup with orphaned services */
+	if (show_orphan_hosts_hostgroup==TRUE)
+		create_orphan_hosts_hostgroup();
+	if (show_orphan_services_servicegroup==TRUE)
+		create_orphan_services_servicegroup();
+#endif
+
 	return result;
         }
 
@@ -4106,3 +4134,64 @@
 	return OK;
         }
 
+
+
+/* Creates a hostgroup with hosts which are not a member of any hostgroup */
+void create_orphan_hosts_hostgroup() {
+
+    hostgroup *orphan_hostgroup;
+    host *temp_host;
+    hostgroup *temp_hostgroup;
+
+    orphan_hostgroup = add_hostgroup("orphans","Orphan Hosts","","","");
+
+    if (orphan_hostgroup) {
+        for (temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next) {
+            int found = 0;
+            for (temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next) {
+                hostsmember *member;
+                for (member=temp_hostgroup->members;member!=NULL;member=member->next) {
+                    // Remember we have found this host
+                    if (strcmp(member->host_name,temp_host->name)==0)
+                        found++;
+                }
+            }
+            // If we didn't find this host in any hostgroup, this is an orphaned host
+            if (found==0) {
+                add_host_to_hostgroup(orphan_hostgroup,temp_host->name);
+            }
+        }
+    }
+
+}
+
+
+/* Creates a servicegroup with services which are not a member of any servicegroup */
+void create_orphan_services_servicegroup() {
+
+    servicegroup *orphan_servicegroup;
+    service *temp_service;
+    servicegroup *temp_servicegroup;
+
+    orphan_servicegroup = add_servicegroup("orphans","Orphan Services","","","");
+
+    if (orphan_servicegroup) {
+        for (temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next) {
+            int found = 0;
+            for (temp_servicegroup=servicegroup_list;temp_servicegroup!=NULL;temp_servicegroup=temp_servicegroup->next) {
+                servicesmember *member;
+                for (member=temp_servicegroup->members;member!=NULL;member=member->next) {
+                    // Remember we have found this host
+                    if (strcmp(member->service_description,temp_service->description)==0)
+                        found++;
+                }
+            }
+            // If we didn't find this service in any servicegroup, this is an orphaned service
+            if (found==0) {
+                add_service_to_servicegroup(orphan_servicegroup,temp_service->host_name,temp_service->description);
+            }
+        }
+    }
+
+}
+
Index: html/docs/configcgi.html
===================================================================
RCS file: /cvsroot/nagios/nagios/html/docs/configcgi.html,v
retrieving revision 1.18
diff -U3 -r1.18 configcgi.html
--- html/docs/configcgi.html	30 Aug 2007 22:35:04 -0000	1.18
+++ html/docs/configcgi.html	10 Aug 2008 17:23:22 -0000
@@ -744,6 +744,55 @@
 This option is used to define the base URL to your Splunk interface.  This URL is used by the CGIs when creating links if the <a href="#enable_splunk_integration">enable_splunk_integration</a> option is enabled.
 </p>
 
+<a name="show_orphan_hosts_hostgroup"></a>
+<table border="0" width="100%" class="Default">
+<tr>
+<td bgcolor="#cbcbcb"><strong>Show Orphan Hosts Hostgroup</strong></td>
+
+</tr>
+</table>
+<br>
+
+<table border="0" class="Default">
+<tr>
+<td valign=top>Format:</td>
+<td><strong>show_orphan_hosts_hostgroup=[0/1]</strong></td>
+</tr>
+<tr>
+<td valign=top>Example:</td>
+<td><font color="red"><strong>show_orphan_hosts_hostgroup=1</strong></font></td>
+</tr>
+
+</table>
+
+<p>
+If set to 1, the CGI programs create an additional "Orphaned Hosts" hostgroup and put all hosts, which are not assigned to any hostgroup, in there. This could be useful if you'd like to determine which hosts you've missed assigning to a hostgroup. You can submit commands to this hostgroup, too. But be aware: the Nagios core only executes this command if create_orphan_hosts in nagios.cfg is set to 1 (see the <a href="configmain.html#create_orphan_hostgroup">Create Orphan Hostgroup setting</a>). Otherwise the submitted command is not executed and there is no warning displayed.
+</p>
+
+<a name="show_orphan_services_servicegroup"></a>
+<table border="0" width="100%" class="Default">
+<tr>
+<td bgcolor="#cbcbcb"><strong>Show Orphan Services Servicegroup</strong></td>
+</tr>
+</table>
+<br>
+
+<table border="0" class="Default">
+<tr>
+
+<td valign=top>Format:</td>
+<td><strong>show_orphan_services_servicegroup=[0/1]</strong></td>
+</tr>
+<tr>
+<td valign=top>Example:</td>
+<td><font color="red"><strong>show_orphan_services_servicegroup=1</strong></font></td>
+</tr>
+</table>
+
+<p>
+If set to 1, the CGI programs create an additional "Orphaned Services" servicegroup and put all services, which are not assigned to any servicegroup, in there. This could be useful if you'd like to determine which services you've missed assigning to a servicegroup. You can submit commands to this servicegroup, too. But be aware: the Nagios core only executes this command if create_orphan_services in nagios.cfg is set to 1 (see the <a href="configmain.html#create_orphan_servicegroup">Create Orphan Servicegroup setting</a>). Otherwise the submitted command is not executed and there is no warning displayed.
+</p>
+
 
 <hr>
 
Index: html/docs/configmain.html
===================================================================
RCS file: /cvsroot/nagios/nagios/html/docs/configmain.html,v
retrieving revision 1.42
diff -U3 -r1.42 configmain.html
--- html/docs/configmain.html	28 Feb 2008 22:33:01 -0000	1.42
+++ html/docs/configmain.html	10 Aug 2008 17:23:25 -0000
@@ -3604,6 +3604,56 @@
 </p>
 
 
+<a name="create_orphan_hostgroup"></a>
+<table border="0" width="100%" class="Default">
+<tr>
+<td bgcolor="#cbcbcb"><strong>Create Orphan Hostgroup</strong></td>
+</tr>
+</table>
+<br>
+
+
+<table border="0" class="Default">
+<tr>
+<td>Format:</td>
+<td><strong>create_orphan_hostgroup=&lt;0/1&gt;</strong></td>
+</tr>
+<tr>
+<td>Example:</td>
+<td><font color="red"><strong>create_orphan_hostgroup=1</strong></font></td>
+</tr>
+</table>
+
+<p>
+If set to 1, Nagios creates an additional "Orphaned Hosts" hostgroup and puts all hosts, which are not assigned to any hostgroup, in there. Commands can be addressed to the hostgroup &quot;orphans&quot;, then. This setting doesn't influence the display of the orphan hostgroup in the CGI programs. See the <a href="configcgi.html#show_orphan_hosts_hostgroup">Show Orphan Hosts Hostgroup setting</a> in the CGI configuration file.
+</p>
+
+
+<a name="create_orphan_servicegroup"></a>
+<table border="0" width="100%" class="Default">
+<tr>
+<td bgcolor="#cbcbcb"><strong>Create Orphan Servicegroup</strong></td>
+</tr>
+</table>
+<br>
+
+
+<table border="0" class="Default">
+<tr>
+<td>Format:</td>
+<td><strong>create_orphan_servicegroup=&lt;0/1&gt;</strong></td>
+</tr>
+<tr>
+<td>Example:</td>
+<td><font color="red"><strong>create_orphan_servicegroup=1</strong></font></td>
+</tr>
+</table>
+
+<p>
+If set to 1, Nagios creates an additional "Orphaned Services" servicegroup and puts all services, which are not assigned to any servicegroup, in there. Commands can be addressed to the servicegroup &quot;orphans&quot;, then. This setting doesn't influence the display of the orphan servicegroup in the CGI programs. See the <a href="configcgi.html#show_orphan_services_servicegroup">Show Orphan Services Servicegroup setting</a> in the CGI configuration file.
+</p>
+
+
 <hr>
 
 </body>
Index: sample-config/cgi.cfg.in
===================================================================
RCS file: /cvsroot/nagios/nagios/sample-config/cgi.cfg.in,v
retrieving revision 1.18
diff -U3 -r1.18 cgi.cfg.in
--- sample-config/cgi.cfg.in	7 Oct 2007 19:42:52 -0000	1.18
+++ sample-config/cgi.cfg.in	10 Aug 2008 17:23:27 -0000
@@ -319,3 +319,25 @@
 #splunk_url=http://127.0.0.1:8000/
 
 
+
+# SHOW ORPHAN HOSTS HOSTGROUP
+# This option controls whether or not the CGI programs create an
+# extra hostgroup named "Orphaned Hosts" and put all hosts, which
+# are not assigned to any hostgroup, in there.
+# Values: 0 = Do not show the orphans hostgroup (default)
+#         1 = Show the hostgroup "orphans" with alias "Orphaned Hosts"
+
+#show_orphan_hosts_hostgroup=1
+
+
+
+# SHOW ORPHAN SERVICES SERVICEGROUP
+# This option controls whether or not the CGI programs create an
+# extra servicegroup named "Orphaned Services" and put all services, which
+# are not assigned to any servicegroup, in there.
+# Values: 0 = Do not show the orphans servicegroup (default)
+#         1 = Show the servicegroup "orphans" with alias "Orphaned Services"
+
+#show_orphan_services_servicegroup=1
+
+
Index: sample-config/nagios.cfg.in
===================================================================
RCS file: /cvsroot/nagios/nagios/sample-config/nagios.cfg.in,v
retrieving revision 1.67
diff -U3 -r1.67 nagios.cfg.in
--- sample-config/nagios.cfg.in	19 May 2008 18:02:08 -0000	1.67
+++ sample-config/nagios.cfg.in	10 Aug 2008 17:23:28 -0000
@@ -1287,3 +1287,27 @@
 max_debug_file_size=1000000
 
 
+
+# CREATE ORPHAN HOSTGROUP
+# This option controls whether or not Nagios creates an extra hostgroup
+# named "Orphaned Hosts" and puts all hosts, which are not assigned
+# to any hostgroup, in there. If enabled, commands can be addressed to 
+# this hostgroup, too.
+# Values: 0 = Do not create the orphans hostgroup (default)
+#         1 = Create the hostgroup "orphans" with alias "Orphaned Hosts"
+
+#create_orphan_hostgroup=1
+
+
+
+# CREATE ORPHAN SERVICEGROUP
+# This option controls whether or not Nagios creates an extra servicegroup
+# named "Orphaned Services" and puts all services, which are not assigned
+# to any servicegroup, in there. If enabled, commands can be addressed to 
+# this servicegroup, too.
+# Values: 0 = Do not create the orphans servicegroup (default)
+#         1 = Create the servicegroup "orphans" with alias "Orphaned Services"
+
+#create_orphan_servicegroup=1
+
+
